# definition of the linguistic annotation steps

ALVISTMP	= $HOME/tmp

PLATFORM_ROOT	= $HOME/Recherche/Projets/ALVIS/Programs/WP5/Platform

AWK		= /bin/awk

<alvis_connection>
	HARVESTER_PORT	= 6234
	NEXTSTEP	= 0
	NEXTSTEP_HOST	= localhost
	NEXTSTEP_PORT	= 7345
	SPOOLDIR	= $HOME/tmp/spool
	OUTDIR		= $HOME/tmp/Outdir
</alvis_connection>

<NLP_connection>
	SERVER			= localhost
	PORT			= 1510
	RETRY_CONNECTION	= 10
</NLP_connection>


<linguistic_annotation>
	ENABLE_TOKEN	= 1
	ENABLE_NER	= 0
	ENABLE_WORD	= 0
	ENABLE_SENTENCE = 0
	ENABLE_POS	= 0
	ENABLE_LEMMA	= 0
	ENABLE_TERM_TAG	= 0
	ENABLE_SYNTAX	= 0
</linguistic_annotation>

<NLP_misc>
	NLP_resources	= $PLATFORM_ROOT/res
	SAVE_IN_OUTDIR	= 1
	TERM_LIST_EN	= $NLP_resources/terms.txt.sav
	TERM_LIST_FR	= $NLP_resources/terms_FR.txt
</NLP_misc>

NLP_tools_root = $PLATFORM_ROOT/tools

<NLP_tools>
	NETAG_EN		= "$NLP_tools_root/TagEN/tagen -t :bio"
	NETAG_FR		= "$NLP_tools_root/TagEN/tagen -t :equer"
	WORDSEG_EN		= "$AWK -f $NLP_tools_root/WordSeg/words-en.awk"
	# WORDSEG_EN		= "$NLP_tools_root/WordSeg/sentences.pl"
	WORDSEG_FR		= "$AWK -f $NLP_tools_root/WordSeg/words-fr.awk"
	POSTAG_EN		= "$NLP_tools_root/TreeTagger/bin/tree-tagger $NLP_tools_root/TreeTagger/lib/english.par -token -lemma -sgml -no-unknown"
	#POSTAG_EN		= "(cd $NLP_tools_root/geniatagger-2.0.1; sed 's/ /___/g' | tr '\n' ' '| ./geniatagger -nt | cut -f 1,2,3|sed 's/\([^\t]*\t\)\([^\t]*\)\t\([^\t]*\)/\1\3\t\2/g' | sed 's/___/ /g' | grep -v '^$')"
	POSTAG_FR		= "$NLP_tools_root/TreeTagger/bin/tree-tagger $NLP_tools_root/TreeTagger/lib/french.par -token -lemma -sgml -no-unknown"
	# SYNTACTIC_PATH_EN	= "$NLP_tools_root/link-4.1b"
	SYNTACTIC_PATH_EN       = "$NLP_tools_root/biolgForAlvis/biolg-1.1.7b"
	SYNTACTIC_PATH_FR	= ""
	# SYNTACTIC_ANALYSIS_EN	= "cd $SYNTACTIC_PATH_EN ; $SYNTACTIC_PATH_EN/parse"
	SYNTACTIC_ANALYSIS_EN	= "cd $SYNTACTIC_PATH_EN ; $SYNTACTIC_PATH_EN/parse -xmlin"
	SYNTACTIC_ANALYSIS_FR	= ""
	TERM_TAG_FR		= "$NLP_tools_root/TermTagger/FiltragepleinText-vers2.pl"
	TERM_TAG_EN		= "$NLP_tools_root/TermTagger/FiltragepleinText-vers2.pl"
</NLP_tools>



